@@ -164,16 +164,14 @@ static void entangle_camera_automata_class_init(EntangleCameraAutomataClass *kla
     g_signal_new("camera-capture-begin",
                  G_TYPE_FROM_CLASS(klass),
                  G_SIGNAL_RUN_FIRST,
-                 G_STRUCT_OFFSET(EntangleCameraAutomataClass, camera_capture_begin),
-                 NULL, NULL,
+                 0, NULL, NULL,
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE,
                  0);
     g_signal_new("camera-capture-end",
                  G_TYPE_FROM_CLASS(klass),
                  G_SIGNAL_RUN_FIRST,
-                 G_STRUCT_OFFSET(EntangleCameraAutomataClass, camera_capture_end),
-                 NULL, NULL,
+                 0, NULL, NULL,
                  g_cclosure_marshal_VOID__VOID,
                  G_TYPE_NONE,
                  0);
@@ -255,7 +253,7 @@ static void do_entangle_camera_file_download(EntangleCamera *cam G_GNUC_UNUSED,
     ENTANGLE_DEBUG("Saved to %s", localpath);
     image = entangle_image_new_file(localpath);
 
-    entangle_session_add(priv->session, image);
+    entangle_session_add_media(priv->session, ENTANGLE_MEDIA(image));
 
     g_object_unref(image);
 
