@@ -172,8 +172,7 @@ static void entangle_camera_list_class_init(EntangleCameraListClass *klass)
     g_signal_new("camera-added",
                  G_TYPE_FROM_CLASS(klass),
                  G_SIGNAL_RUN_FIRST,
-                 G_STRUCT_OFFSET(EntangleCameraListClass, camera_added),
-                 NULL, NULL,
+                 0, NULL, NULL,
                  g_cclosure_marshal_VOID__OBJECT,
                  G_TYPE_NONE,
                  1,
@@ -182,8 +181,7 @@ static void entangle_camera_list_class_init(EntangleCameraListClass *klass)
     g_signal_new("camera-removed",
                  G_TYPE_FROM_CLASS(klass),
                  G_SIGNAL_RUN_FIRST,
-                 G_STRUCT_OFFSET(EntangleCameraListClass, camera_removed),
-                 NULL, NULL,
+                 0, NULL, NULL,
                  g_cclosure_marshal_VOID__OBJECT,
                  G_TYPE_NONE,
                  1,
@@ -508,7 +506,7 @@ void entangle_camera_list_remove(EntangleCameraList *list,
 /**
  * entangle_camera_list_get:
  * @list: (transfer none): the camera list
- * @entry: (transfer none): the index of the camera to get
+ * @entry: the index of the camera to get
  *
  * Get the camera at position @entry in the list
  *
