@@ -47,19 +47,12 @@ struct _EntangleDeviceManager
 struct _EntangleDeviceManagerClass
 {
     GObjectClass parent_class;
-
-    void (*device_added)(EntangleDeviceManager *manager, const char *port);
-    void (*device_removed)(EntangleDeviceManager *manager, const char *port);
 };
 
 
 GType entangle_device_manager_get_type(void) G_GNUC_CONST;
 EntangleDeviceManager* entangle_device_manager_new(void);
 
-gboolean entangle_device_manager_has_port(EntangleDeviceManager *manager, const char *port);
-
-char *entangle_device_manager_port_serial(EntangleDeviceManager *manager, const char *port);
-
 G_END_DECLS
 
 #endif /* __ENTANGLE_DEVICE_MANAGER_H__ */
