@@ -24,6 +24,7 @@
 
 #include "entangle-debug.h"
 #include "entangle-session-browser.h"
+#include "entangle-video.h"
 
 #define ENTANGLE_SESSION_BROWSER_GET_PRIVATE(obj)                       \
     (G_TYPE_INSTANCE_GET_PRIVATE((obj), ENTANGLE_TYPE_SESSION_BROWSER, EntangleSessionBrowserPrivate))
@@ -36,17 +37,18 @@ struct _EntangleSessionBrowserPrivate {
     GdkRGBA background;
     GdkRGBA highlight;
 
-    gulong sigImageAdded;
-    gulong sigImageRemoved;
+    gulong sigMediaAdded;
+    gulong sigMediaRemoved;
     gulong sigThumbReady;
 
-    EntangleImage *selectedImage;
+    EntangleMedia *selectedMedia;
     gint selectedIndex;
 
     GtkAdjustment *hadjustment;
     GtkAdjustment *vadjustment;
 
     GdkPixbuf *imageIcon;
+    GdkPixbuf *videoIcon;
 
     guint hscroll_policy : 1;
     guint vscroll_policy : 1;
@@ -122,7 +124,7 @@ entangle_session_browser_scroll_to_selected(EntangleSessionBrowser *browser)
         return;
 
     minx = priv->selectedIndex * (priv->margin + priv->itemsize);
-    maxx = minx + priv->itemsize;
+    maxx = minx + priv->itemsize + (priv->margin * 2);
 
     gdk_window_get_position(priv->bin_window, &x, &y);
     gtk_widget_get_allocation(GTK_WIDGET(browser), &allocation);
@@ -140,29 +142,29 @@ entangle_session_browser_scroll_to_selected(EntangleSessionBrowser *browser)
 
 
 static gboolean
-entangle_session_browser_select_image(EntangleSessionBrowser *browser,
+entangle_session_browser_select_media(EntangleSessionBrowser *browser,
                                       int idx)
 {
     EntangleSessionBrowserPrivate *priv = browser->priv;
-    EntangleImage *img;
+    EntangleMedia *media;
 
     if (idx == -1) {
-        img = NULL;
+        media = NULL;
     } else {
-        img = entangle_session_image_get(priv->session, idx);
-        if (!img) {
+        media = entangle_session_get_media(priv->session, idx);
+        if (!media) {
             idx = -1;
         }
     }
     if (idx != priv->selectedIndex ||
-        img != priv->selectedImage) {
-        if (priv->selectedImage) {
-            g_object_unref(priv->selectedImage);
+        media != priv->selectedMedia) {
+        if (priv->selectedMedia) {
+            g_object_unref(priv->selectedMedia);
         }
-        priv->selectedImage = img;
+        priv->selectedMedia = media;
         priv->selectedIndex = idx;
-        if (img) {
-            g_object_ref(priv->selectedImage);
+        if (media) {
+            g_object_ref(priv->selectedMedia);
         }
         entangle_session_browser_scroll_to_selected(browser);
         g_signal_emit(browser, browser_signals[SIGNAL_SELECTION_CHANGED], 0);
@@ -183,37 +185,41 @@ static void do_thumb_loaded(EntanglePixbufLoader *loader G_GNUC_UNUSED,
 }
 
 
-static void do_image_added(EntangleSession *session G_GNUC_UNUSED,
-                           EntangleImage *img,
+static void do_media_added(EntangleSession *session G_GNUC_UNUSED,
+                           EntangleMedia *media,
                            gpointer data)
 {
     EntangleSessionBrowser *browser = data;
     EntangleSessionBrowserPrivate *priv = browser->priv;
 
-    ENTANGLE_DEBUG("Request image %s for new image", entangle_image_get_filename(img));
-    entangle_pixbuf_loader_load(ENTANGLE_PIXBUF_LOADER(priv->loader), img);
+    ENTANGLE_DEBUG("Request image %s for new image", entangle_media_get_filename(media));
+    if (ENTANGLE_IS_IMAGE(media))
+        entangle_pixbuf_loader_load(ENTANGLE_PIXBUF_LOADER(priv->loader),
+                                    ENTANGLE_IMAGE(media));
 
-    entangle_session_browser_select_image(browser,
-                                          entangle_session_image_count(priv->session) - 1);
+    entangle_session_browser_select_media(browser,
+                                          entangle_session_get_media_count(priv->session) - 1);
     gtk_widget_queue_resize(GTK_WIDGET(browser));
 }
 
 
 
-static void do_image_removed(EntangleSession *session G_GNUC_UNUSED,
-                             EntangleImage *img,
+static void do_media_removed(EntangleSession *session G_GNUC_UNUSED,
+                             EntangleMedia *media,
                              gpointer data)
 {
     EntangleSessionBrowser *browser = data;
     EntangleSessionBrowserPrivate *priv = browser->priv;
 
-    ENTANGLE_DEBUG("Unrequest image %s for new image", entangle_image_get_filename(img));
-    entangle_pixbuf_loader_unload(ENTANGLE_PIXBUF_LOADER(priv->loader), img);
+    ENTANGLE_DEBUG("Unrequest image %s for new image", entangle_media_get_filename(media));
+    if (ENTANGLE_IS_IMAGE(media))
+        entangle_pixbuf_loader_unload(ENTANGLE_PIXBUF_LOADER(priv->loader),
+                                      ENTANGLE_IMAGE(media));
 
     if (priv->selectedIndex >= 0) {
-        img = entangle_session_image_get(priv->session, priv->selectedIndex);
-        if (img != priv->selectedImage) {
-            entangle_session_browser_select_image(browser, priv->selectedIndex - 1);
+        media = entangle_session_get_media(priv->session, priv->selectedIndex);
+        if (media != priv->selectedMedia) {
+            entangle_session_browser_select_media(browser, priv->selectedIndex - 1);
         }
     }
 
@@ -231,21 +237,23 @@ static void do_model_unload(EntangleSessionBrowser *browser)
     ENTANGLE_DEBUG("Unload model");
 
     g_signal_handler_disconnect(priv->session,
-                                priv->sigImageAdded);
+                                priv->sigMediaAdded);
     g_signal_handler_disconnect(priv->session,
-                                priv->sigImageRemoved);
+                                priv->sigMediaRemoved);
     g_signal_handler_disconnect(priv->loader,
                                 priv->sigThumbReady);
 
-    count = entangle_session_image_count(priv->session);
+    count = entangle_session_get_media_count(priv->session);
     for (int i = 0; i < count; i++) {
-        EntangleImage *img = entangle_session_image_get(priv->session, i);
-        entangle_pixbuf_loader_unload(ENTANGLE_PIXBUF_LOADER(priv->loader), img);
+        EntangleMedia *media = entangle_session_get_media(priv->session, i);
+        if (ENTANGLE_IS_IMAGE(media))
+            entangle_pixbuf_loader_unload(ENTANGLE_PIXBUF_LOADER(priv->loader),
+                                          ENTANGLE_IMAGE(media));
     }
     priv->selectedIndex = -1;
-    if (priv->selectedImage)
-        g_object_unref(priv->selectedImage);
-    priv->selectedImage = NULL;
+    if (priv->selectedMedia)
+        g_object_unref(priv->selectedMedia);
+    priv->selectedMedia = NULL;
 }
 
 static void do_model_load(EntangleSessionBrowser *browser)
@@ -257,22 +265,24 @@ static void do_model_load(EntangleSessionBrowser *browser)
 
     ENTANGLE_DEBUG("Load model");
 
-    priv->sigImageAdded = g_signal_connect(priv->session, "session-image-added",
-                                           G_CALLBACK(do_image_added), browser);
-    priv->sigImageRemoved = g_signal_connect(priv->session, "session-image-removed",
-                                             G_CALLBACK(do_image_removed), browser);
+    priv->sigMediaAdded = g_signal_connect(priv->session, "session-media-added",
+                                           G_CALLBACK(do_media_added), browser);
+    priv->sigMediaRemoved = g_signal_connect(priv->session, "session-media-removed",
+                                             G_CALLBACK(do_media_removed), browser);
     priv->sigThumbReady = g_signal_connect(priv->loader, "pixbuf-loaded",
                                            G_CALLBACK(do_thumb_loaded), browser);
 
-    count = entangle_session_image_count(priv->session);
+    count = entangle_session_get_media_count(priv->session);
     for (int i = 0; i < count; i++) {
-        EntangleImage *img = entangle_session_image_get(priv->session, i);
+        EntangleMedia *media = entangle_session_get_media(priv->session, i);
 
-        entangle_pixbuf_loader_load(ENTANGLE_PIXBUF_LOADER(priv->loader), img);
+        if (ENTANGLE_IS_IMAGE(media))
+            entangle_pixbuf_loader_load(ENTANGLE_PIXBUF_LOADER(priv->loader),
+                                        ENTANGLE_IMAGE(media));
     }
 
     if (count) {
-        entangle_session_browser_select_image(browser, count - 1);
+        entangle_session_browser_select_media(browser, count - 1);
     }
 }
 
@@ -379,7 +389,7 @@ entangle_session_browser_realize(GtkWidget *widget)
 
     if (priv->session) {
         priv->binwidth = (priv->itemsize + priv->margin) *
-            entangle_session_image_count(priv->session) +
+            entangle_session_get_media_count(priv->session) +
             priv->margin;
     } else {
         priv->binwidth = allocation.width;
@@ -453,7 +463,7 @@ static int entangle_session_browser_get_index_at_coords(EntangleSessionBrowser *
         return -1;
 
     idx = x / (priv->itemsize + priv->margin);
-    if (idx >= entangle_session_image_count(priv->session))
+    if (idx >= entangle_session_get_media_count(priv->session))
         return -1;
 
     itemx = (idx * (priv->itemsize + priv->margin)) + priv->margin;
@@ -465,16 +475,16 @@ static int entangle_session_browser_get_index_at_coords(EntangleSessionBrowser *
 
 
 /**
- * entangle_session_browser_get_image_at_coords:
+ * entangle_session_browser_get_media_at_coords:
  * @browser: (transfer none): the session browser
  * @x: the horizontal co-ordinate
  * @y: the vertical co-ordinate
  *
- * Retrieve the image displayed at the co-ordinates (@x, @y)
+ * Retrieve the media displayed at the co-ordinates (@x, @y)
  *
- * Returns: (transfer none): the image, or NULL
+ * Returns: (transfer none): the media, or NULL
  */
-EntangleImage *entangle_session_browser_get_image_at_coords(EntangleSessionBrowser *browser,
+EntangleMedia *entangle_session_browser_get_media_at_coords(EntangleSessionBrowser *browser,
                                                             gint x, gint y)
 {
     g_return_val_if_fail(ENTANGLE_IS_SESSION_BROWSER(browser), NULL);
@@ -483,7 +493,7 @@ EntangleImage *entangle_session_browser_get_image_at_coords(EntangleSessionBrows
     int idx = entangle_session_browser_get_index_at_coords(browser, x, y);
 
     if (idx >= 0)
-        return entangle_session_image_get(priv->session, idx);
+        return entangle_session_get_media(priv->session, idx);
     else
         return NULL;
 }
@@ -514,7 +524,7 @@ entangle_session_browser_button_press(GtkWidget *widget,
                                                            event->y);
 
         if (idx >= 0 &&
-            entangle_session_browser_select_image(browser, idx)) {
+            entangle_session_browser_select_media(browser, idx)) {
             priv->dnd_start_x = event->x;
             priv->dnd_start_y = event->y;
         }
@@ -580,17 +590,17 @@ entangle_session_browser_query_tooltip(GtkWidget *widget,
     gint bx = 0;
     gint by = 0;
     GdkRectangle rect;
-    EntangleImage *img;
+    EntangleMedia *media;
     const gchar *filename;
 
     entangle_session_browser_convert_widget_to_bin_window_coords(browser,
                                                                  x, y,
                                                                  &bx, &by);
 
-    img = entangle_session_browser_get_image_at_coords(browser,
+    media = entangle_session_browser_get_media_at_coords(browser,
                                                        bx, by);
 
-    if (img == NULL) {
+    if (media == NULL) {
         return FALSE;
     }
 
@@ -599,7 +609,7 @@ entangle_session_browser_query_tooltip(GtkWidget *widget,
     rect.height = priv->itemsize;
     rect.width = priv->itemsize;
 
-    filename = entangle_image_get_filename(img);
+    filename = entangle_media_get_filename(media);
     if (filename) {
         gtk_tooltip_set_text(tooltip, filename);
         gtk_tooltip_set_tip_area(tooltip, &rect);
@@ -650,13 +660,13 @@ entangle_session_browser_motion_notify(GtkWidget *widget,
 
     EntangleSessionBrowser *browser = ENTANGLE_SESSION_BROWSER(widget);
     EntangleSessionBrowserPrivate *priv = browser->priv;
-    GdkPixbuf *pixbuf;
+    GdkPixbuf *pixbuf = NULL;
     GtkTargetEntry tentry[] = {
         { g_strdup("demo"), GTK_TARGET_SAME_APP, 0,}
     };
     GtkTargetList *tlist;
     GdkDragContext *ctx;
-    EntangleImage *img;
+    EntangleMedia *media;
 
     if (priv->dnd_start_y == -1 || priv->dnd_start_x == -1)
         return FALSE;
@@ -667,42 +677,37 @@ entangle_session_browser_motion_notify(GtkWidget *widget,
                                   event->x, event->y))
         return TRUE;
 
-    img = entangle_session_browser_get_image_at_coords(browser,
-                                                       priv->dnd_start_x,
-                                                       priv->dnd_start_y);
+    media = entangle_session_browser_get_media_at_coords(browser,
+                                                         priv->dnd_start_x,
+                                                         priv->dnd_start_y);
 
-    if (!img) {
+    if (!media) {
         priv->dnd_start_x = priv->dnd_start_y = -1;
         return FALSE;
     }
 
-    pixbuf = entangle_pixbuf_loader_get_pixbuf(ENTANGLE_PIXBUF_LOADER(priv->loader),
-                                               img);
+    if (ENTANGLE_IS_IMAGE(media)) {
+        pixbuf = entangle_pixbuf_loader_get_pixbuf(ENTANGLE_PIXBUF_LOADER(priv->loader),
+                                                   ENTANGLE_IMAGE(media));
 
-    if (!pixbuf) {
-        priv->dnd_start_x = priv->dnd_start_y = -1;
-        return FALSE;
+        if (!pixbuf) {
+            priv->dnd_start_x = priv->dnd_start_y = -1;
+            return FALSE;
+        }
     }
 
     tlist = gtk_target_list_new(tentry, G_N_ELEMENTS(tentry));
 
-#if GTK_CHECK_VERSION(3,10,0)
     ctx = gtk_drag_begin_with_coordinates(widget,
                                           tlist,
                                           GDK_ACTION_PRIVATE,
                                           1,
                                           (GdkEvent*)event,
                                           -1, -1);
-#else
-    ctx = gtk_drag_begin(widget,
-                         tlist,
-                         GDK_ACTION_PRIVATE,
-                         1,
-                         (GdkEvent*)event);
-#endif
 
     priv->dnd_start_x = priv->dnd_start_y = -1;
-    gtk_drag_set_icon_pixbuf(ctx, pixbuf, 0, 0);
+    if (pixbuf)
+        gtk_drag_set_icon_pixbuf(ctx, pixbuf, 0, 0);
 
     gtk_target_list_unref(tlist);
 
@@ -721,14 +726,14 @@ entangle_session_browser_key_press(GtkWidget *widget,
     switch (event->keyval) {
     case GDK_KEY_Right:
         if (priv->selectedIndex != -1 &&
-            ((priv->selectedIndex + 1) < entangle_session_image_count(priv->session))) {
-            entangle_session_browser_select_image(browser, priv->selectedIndex + 1);
+            ((priv->selectedIndex + 1) < entangle_session_get_media_count(priv->session))) {
+            entangle_session_browser_select_media(browser, priv->selectedIndex + 1);
         }
         return TRUE;
 
     case GDK_KEY_Left:
         if (priv->selectedIndex > 0) {
-            entangle_session_browser_select_image(browser, priv->selectedIndex - 1);
+            entangle_session_browser_select_media(browser, priv->selectedIndex - 1);
         }
         return TRUE;
 
@@ -753,7 +758,7 @@ entangle_session_browser_size_allocate(GtkWidget *widget,
 
     if (priv->session) {
         priv->binwidth = (priv->itemsize + priv->margin) *
-            entangle_session_image_count(priv->session) +
+            entangle_session_get_media_count(priv->session) +
             priv->margin;
     } else {
         priv->binwidth = allocation->width;
@@ -818,6 +823,8 @@ static void entangle_session_browser_finalize(GObject *object)
         g_object_unref(priv->loader);
     if (priv->imageIcon)
         g_object_unref(priv->imageIcon);
+    if (priv->videoIcon)
+        g_object_unref(priv->videoIcon);
 
     G_OBJECT_CLASS(entangle_session_browser_parent_class)->finalize(object);
 }
@@ -875,8 +882,7 @@ static void entangle_session_browser_class_init(EntangleSessionBrowserClass *kla
         g_signal_new("selection-changed",
                      G_TYPE_FROM_CLASS(object_class),
                      G_SIGNAL_RUN_FIRST,
-                     G_STRUCT_OFFSET(EntangleSessionBrowserClass, selection_changed),
-                     NULL, NULL,
+                     0, NULL, NULL,
                      g_cclosure_marshal_VOID__VOID,
                      G_TYPE_NONE, 0);
 
@@ -903,6 +909,13 @@ static void entangle_session_browser_load_icons(EntangleSessionBrowser *browser)
                                                256,
                                                0,
                                                NULL);
+    if (priv->videoIcon)
+        g_object_unref(priv->videoIcon);
+    priv->videoIcon = gtk_icon_theme_load_icon(theme,
+                                               "camera-video-symbolic",
+                                               256,
+                                               0,
+                                               NULL);
 }
 
 
@@ -936,27 +949,27 @@ static void entangle_session_browser_init(EntangleSessionBrowser *browser)
 
 
 /**
- * entangle_session_browser_selected_image:
+ * entangle_session_browser_get_selected_media:
  * @browser: (transfer none): the session browser
  *
- * Returns: (transfer full): the selected image or NULL
+ * Returns: (transfer full): the selected media or NULL
  */
-EntangleImage *entangle_session_browser_selected_image(EntangleSessionBrowser *browser)
+EntangleMedia *entangle_session_browser_get_selected_media(EntangleSessionBrowser *browser)
 {
     g_return_val_if_fail(ENTANGLE_IS_SESSION_BROWSER(browser), NULL);
 
     EntangleSessionBrowserPrivate *priv = browser->priv;
 
-    if (priv->selectedImage == NULL)
+    if (priv->selectedMedia == NULL)
         return NULL;
 
-    g_object_ref(priv->selectedImage);
-    return priv->selectedImage;
+    g_object_ref(priv->selectedMedia);
+    return priv->selectedMedia;
 }
 
 
 /**
- * entangle_session_browser_earlier_images:
+ * entangle_session_browser_get_earlier_images:
  * @browser: (transfer none): the session browser
  * @include_selected: true to include the current image in the list
  * @count: maximum number of images to return
@@ -967,9 +980,9 @@ EntangleImage *entangle_session_browser_selected_image(EntangleSessionBrowser *b
  *
  * Returns: (transfer full)(element-type EntangleImage): the list of images
  */
-GList *entangle_session_browser_earlier_images(EntangleSessionBrowser *browser,
-                                               gboolean include_selected,
-                                               gsize count)
+GList *entangle_session_browser_get_earlier_images(EntangleSessionBrowser *browser,
+                                                   gboolean include_selected,
+                                                   gsize count)
 {
     g_return_val_if_fail(ENTANGLE_IS_SESSION_BROWSER(browser), NULL);
 
@@ -985,10 +998,12 @@ GList *entangle_session_browser_earlier_images(EntangleSessionBrowser *browser,
         idx--;
 
     while (idx >= 0 && count > 0) {
-        EntangleImage *img = entangle_session_image_get(priv->session, idx);
-        g_object_ref(img);
-        images = g_list_append(images, img);
-        count--;
+        EntangleMedia *media = entangle_session_get_media(priv->session, idx);
+        if (ENTANGLE_IS_IMAGE(media)) {
+            g_object_ref(media);
+            images = g_list_append(images, media);
+            count--;
+        }
         idx--;
     }
     return images;
@@ -1163,8 +1178,8 @@ entangle_session_browser_draw(GtkWidget *widget,
     if (!priv->session)
         return TRUE;
 
-    for (i = 0; i < entangle_session_image_count(priv->session); i++) {
-        EntangleImage *img = entangle_session_image_get(priv->session, i);
+    for (i = 0; i < entangle_session_get_media_count(priv->session); i++) {
+        EntangleMedia *media = entangle_session_get_media(priv->session, i);
         GdkRectangle paint_area;
 
         paint_area.x = (priv->itemsize + priv->margin) * i + priv->margin;
@@ -1172,7 +1187,7 @@ entangle_session_browser_draw(GtkWidget *widget,
         paint_area.width  = priv->itemsize;
         paint_area.height = priv->itemsize;
 
-        if (img == priv->selectedImage) {
+        if (media == priv->selectedMedia) {
             cairo_save(cr);
             cairo_set_source_rgba(cr, priv->highlight.red, priv->highlight.green, priv->highlight.blue, 1);
             cairo_rectangle(cr, paint_area.x, paint_area.y, paint_area.width, paint_area.height);
@@ -1185,12 +1200,22 @@ entangle_session_browser_draw(GtkWidget *widget,
         cairo_clip(cr);
 
         if (gdk_cairo_get_clip_rectangle(cr, NULL)) {
-            GdkPixbuf *pixbuf = entangle_pixbuf_loader_get_pixbuf(
-                ENTANGLE_PIXBUF_LOADER(priv->loader), img);
+            GdkPixbuf *pixbuf = NULL;
             gint itemsize = priv->itemsize;
             gdouble offset = 0;
-            if (!pixbuf) {
-                pixbuf = priv->imageIcon;
+            if (ENTANGLE_IS_IMAGE(media)) {
+                pixbuf = entangle_pixbuf_loader_get_pixbuf(ENTANGLE_PIXBUF_LOADER(priv->loader),
+                                                           ENTANGLE_IMAGE(media));
+                if (!pixbuf) {
+                    pixbuf = priv->imageIcon;
+                    itemsize /= 2;
+                    offset = itemsize / 2.0;
+                } else {
+                    itemsize -= 2;
+                    offset = 1;
+                }
+            } else if (ENTANGLE_IS_VIDEO(media)) {
+                pixbuf = priv->videoIcon;
                 itemsize /= 2;
                 offset = itemsize / 2.0;
             }
